@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 from typing import TYPE_CHECKING
 
 import logging
@@ -44,7 +43,9 @@ from gajim.common.helpers import play_sound
 from gajim.common.helpers import show_in_folder
 from gajim.common.i18n import _
 from gajim.common.modules.bytestream import is_transfer_active
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.plugins.manifest import PluginManifest
 from gajim.plugins.repository import PluginRepository
 
@@ -121,6 +122,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self.connect('notify::is-active', self._on_window_active)
         self.connect('delete-event', self._on_window_delete)
         self.connect('window-state-event', self._on_window_state_changed)
+        self.connect_after('key-press-event', self._on_key_press_event)
 
         self._ui.connect_signals(self)
 
@@ -165,7 +167,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def get_action(self, name: str) -> Gio.SimpleAction:
         action = self.lookup_action(name)
-        assert action is not None
+        assert isinstance(action, Gio.SimpleAction)
         return action
 
     def get_chat_stack(self) -> ChatStack:
@@ -249,6 +251,42 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def update_account_unread_count(self, account: str, count: int) -> None:
         self._account_side_bar.update_unread_count(account, count)
 
+    def _on_key_press_event(
+        self,
+        _window: MainWindow,
+        event: Gdk.EventKey
+    ) -> bool:
+
+        modifier = event.get_state() & Gtk.accelerator_get_default_mod_mask()
+
+        if not modifier:
+            if event.keyval in (
+                Gdk.KEY_Control_L,
+                Gdk.KEY_Control_R,
+                Gdk.KEY_Alt_L,
+                Gdk.KEY_Alt_R,
+            ):
+                return False
+
+        if modifier == Gdk.ModifierType.CONTROL_MASK:
+            if event.keyval == Gdk.KEY_C:
+                return False
+
+        focused_widget = self.get_focus()
+        if (isinstance(focused_widget, Gtk.TextView)
+            and focused_widget.props.editable):
+                return False
+
+        if isinstance(focused_widget, Gtk.Entry):
+            return False
+
+        message_input = self.get_chat_stack().get_message_input()
+        if not message_input.get_mapped() or not message_input.is_sensitive():
+            return False
+
+        message_input.grab_focus()
+        return self.propagate_key_event(event)
+
     def _on_client_state_changed(self,
                                  client: Client,
                                  _signal_name: str,
@@ -262,8 +300,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self.add_app_message(event.name)
 
     def _on_gajim_update_available(self,
-                                   event: events.GajimUpdateAvailable) -> None:
-        self.add_app_message(event.name, event.version)
+                                   event: events.GajimUpdateAvailable
+                                   ) -> None:
+
+        self.add_app_message(
+            event.name,
+            new_version=event.version,
+            new_setup_url=event.setup_url)
 
     @staticmethod
     def _on_roster_item_exchange(event: events.RosterItemExchangeEvent) -> None:
@@ -330,7 +373,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         if self.chat_exists(event.account, event.jid):
             return
 
-        self.add_group_chat(event.account, event.jid)
+        self.add_group_chat(event.account, event.jid, select=event.select_chat)
 
     def _on_message_sent(self, event: events.MessageSent) -> None:
         if not event.play_sound:
@@ -410,6 +453,9 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('switch-workspace-7', self._on_action),
             ('switch-workspace-8', self._on_action),
             ('switch-workspace-9', self._on_action),
+            ('increase-app-font-size', self._on_app_font_size_action),
+            ('decrease-app-font-size', self._on_app_font_size_action),
+            ('reset-app-font-size', self._on_app_font_size_action),
             ('toggle-chat-list', self._on_action),
             ('preview-download', self._on_preview_action),
             ('preview-open', self._on_preview_action),
@@ -419,6 +465,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('preview-open-link', self._on_preview_action),
             ('copy-message', self._on_copy_message),
             ('retract-message', self._on_retract_message),
+            ('delete-message-locally', self._on_delete_message_locally),
             ('add-workspace', self._add_workspace),
             ('edit-workspace', self._edit_workspace),
             ('remove-workspace', self._remove_workspace),
@@ -435,7 +482,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   _param: Optional[GLib.Variant]) -> Optional[int]:
+                   _param: GLib.Variant | None) -> int | None:
 
         action_name = action.get_name()
         log.info('Activate action: %s', action_name)
@@ -470,6 +517,9 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                                             control.contact.jid)
                 return None
 
+        if action_name == 'escape' and app.settings.get('escape_key_closes'):
+            self.emit('delete-event', Gdk.Event())
+
         if action_name == 'restore-chat':
             self._chat_page.restore_chat()
 
@@ -498,6 +548,34 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         return None
 
+    def _on_app_font_size_action(
+            self,
+            action: Gio.SimpleAction,
+            _param: GLib.Variant
+            ) -> None:
+
+        action_name = action.get_name()
+        if action_name == 'reset-app-font-size':
+            app.settings.set_app_setting('app_font_size', None)
+            app.css_config.apply_app_font_size()
+            return
+
+        app_font_size = app.settings.get('app_font_size')
+        new_app_font_size = app_font_size
+        if action_name == 'increase-app-font-size':
+            new_app_font_size = app_font_size + 0.125
+        elif action_name == 'decrease-app-font-size':
+            new_app_font_size = app_font_size - 0.125
+
+        # Clamp font size
+        new_app_font_size = max(min(1.5, new_app_font_size), 1.0)
+
+        if new_app_font_size == app_font_size:
+            return
+
+        app.settings.set_app_setting('app_font_size', new_app_font_size)
+        app.css_config.apply_app_font_size()
+
     def _on_preview_action(self,
                            action: Gio.SimpleAction,
                            param: GLib.Variant
@@ -526,6 +604,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         elif action_name == 'preview-save-as':
             def _on_ok(paths: list[str]) -> None:
+                if not paths:
+                    ErrorDialog(
+                        _('Could not save file'),
+                        _('Could not save file to selected directory.'),
+                        transient_for=self)
+                    return
+
                 target = paths[0]
                 assert preview is not None
                 assert preview.orig_path is not None
@@ -644,6 +729,26 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             input_str=_('Spam'),
             transient_for=app.window).show()
 
+    def _on_delete_message_locally(self,
+                                   _action: Gio.SimpleAction,
+                                   param: GLib.Variant
+                                   ) -> None:
+
+        def _on_delete() -> None:
+            log_line_id = param.get_uint32()
+            app.storage.archive.delete_message_from_logs(log_line_id)
+            control = self.get_control()
+            control.remove_message(log_line_id)
+
+        ConfirmationDialog(
+            _('Delete Message'),
+            _('Delete message locally?'),
+            _('This message will be deleted from your local chat history'),
+            [DialogButton.make('Cancel'),
+             DialogButton.make('Delete',
+                               callback=_on_delete)],
+            transient_for=app.window).show()
+
     def _on_window_motion_notify(self,
                                  _widget: Gtk.ApplicationWindow,
                                  _event: Gdk.EventMotion
@@ -729,12 +834,14 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self._account_side_bar.activate_account_page(account)
         self._main_stack.show_account(account)
 
-    def get_active_workspace(self) -> Optional[str]:
+    def get_active_workspace(self) -> str | None:
         return self._workspace_side_bar.get_active_workspace()
 
     def is_chat_active(self, account: str, jid: JID) -> bool:
         if not self.has_toplevel_focus():
             return False
+        if self._main_stack.get_visible_page_name() != 'chats':
+            return False
         return self._chat_page.is_chat_selected(account, jid)
 
     def highlight_dnd_targets(self, drag_row: Any, highlight: bool) -> None:
@@ -769,11 +876,11 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                        param: GLib.Variant) -> None:
 
         workspace_id = param.get_string()
-        if workspace_id is not None:
+        if workspace_id:
             self.add_workspace(workspace_id)
 
     def add_workspace(self,
-                      workspace_id: Optional[str] = None,
+                      workspace_id: str | None = None,
                       switch: bool = True) -> str:
 
         if workspace_id is None:
@@ -860,7 +967,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                             param: GLib.Variant) -> None:
 
         workspace_id = param.get_string()
-        if workspace_id is not None:
+        if workspace_id:
             self.activate_workspace(workspace_id)
 
     def activate_workspace(self, workspace_id: str) -> None:
@@ -939,7 +1046,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                  jid: JID,
                  type_: str,
                  select: bool = False,
-                 message: Optional[str] = None
+                 message: str | None = None
                  ) -> None:
 
         workspace_id = self._get_suitable_workspace(account, jid)
@@ -965,6 +1072,12 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                                                'pm',
                                                select=select)
 
+    def clear_chat_list_row(self, account: str, jid: JID) -> None:
+        chat_list_stack = self._chat_page.get_chat_list_stack()
+        chat_list = chat_list_stack.find_chat(account, jid)
+        if chat_list is not None:
+            chat_list.clear_chat_list_row(account, jid)
+
     def select_chat(self, account: str, jid: JID) -> None:
         self._app_side_bar.unselect_all()
         self._account_side_bar.unselect_all()
@@ -998,8 +1111,11 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def add_app_message(self,
                         category: str,
-                        message: Optional[str] = None) -> None:
-        self._app_page.add_app_message(category, message)
+                        new_version: str | None = None,
+                        new_setup_url: str | None = None
+                        ) -> None:
+
+        self._app_page.add_app_message(category, new_version, new_setup_url)
 
     def get_control(self) -> ChatControl:
         return self._chat_page.get_control()
@@ -1066,6 +1182,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         client.get_module('ChatMarkers').send_displayed_marker(
             contact,
             last_message.message_id,
@@ -1089,7 +1207,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def get_preferred_ft_method(self,
                                 contact: types.ChatContactT
-                                ) -> Optional[str]:
+                                ) -> str | None:
 
         ft_pref = app.settings.get_account_setting(
             contact.account,
@@ -1117,21 +1235,22 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def show_add_join_groupchat(self,
                                 account: str,
                                 jid: str,
-                                nickname: Optional[str] = None,
-                                password: Optional[str] = None
+                                nickname: str | None = None,
+                                password: str | None = None
                                 ) -> None:
 
-        if not self.chat_exists(account, JID.from_string(jid)):
+        jid_ = JID.from_string(jid)
+        if not self.chat_exists(account, jid_):
             client = app.get_client(account)
             client.get_module('MUC').join(
-                jid, nick=nickname, password=password)
+                jid_, nick=nickname, password=password)
 
-        self.add_group_chat(account, JID.from_string(jid), select=True)
+        self.add_group_chat(account, jid_, select=True)
 
     def start_chat_from_jid(self,
                             account: str,
                             jid: str,
-                            message: Optional[str] = None
+                            message: str | None = None
                             ) -> None:
 
         jid_ = JID.from_string(jid)
@@ -1160,12 +1279,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         client = app.get_client(account)
 
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(contact, BareContact)
         if contact.is_blocked:
             client.get_module('Blocking').unblock([jid])
             return
 
         # TODO: Keep "confirm_block" setting?
-        def _block_contact(report: Optional[str] = None) -> None:
+        def _block_contact(report: str | None = None) -> None:
             client.get_module('Blocking').block([contact.jid], report)
             self._chat_page.remove_chat(account, contact.jid)
 
@@ -1192,6 +1312,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             client.get_module('Roster').delete_item(jid)
 
         contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         sec_text = _('You are about to remove %(name)s (%(jid)s) from '
                      'your contact list.\n') % {
                          'name': contact.name,
@@ -1277,7 +1399,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         app.settings.set('mainwin_height', window_height)
         app.settings.save()
 
-        def on_continue2(message: Optional[str]) -> None:
+        def on_continue2(message: str | None) -> None:
             if 'file_transfers' not in app.interface.instances:
                 app.app.start_shutdown(message=message)
                 return
@@ -1305,7 +1427,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                 return
             app.app.start_shutdown(message=message)
 
-        def on_continue(message: Optional[str]) -> None:
+        def on_continue(message: str | None) -> None:
             if message is None:
                 # user pressed Cancel to change status message dialog
                 return
